module.exports = function (moduleName, path, contents) {
	var newContents = '';
	var regex = /^function\s+\$ng\s*\(([^()]*)\)/;
	var lastCopied = 0;

	function addQuotes(x) {
		return '"' + x.trim() + '", ';
	}

	function istag(index) {
		return (index + 10) < contents.length
		    && contents[index] === 'f'
		    && contents[index + 1] === 'u'
		    && contents[index + 2] === 'n'
			  && regex.exec(contents.substr(index));
	}

	function skipStringsAndCommentsAndRegexes(i) {
		if (contents[i] === '/' && i < contents.length - 1) {
			if (contents[i + 1] === '/') {
				while (i < contents.length && contents[i] !== '\n')
					i++;
			} else if (contents[i + 1] === '*') {
				i += 2;

				while (i < contents.length - 1 && contents[i] !== '*' && contents[i + 1] !== '/')
					i++;
			} else { // check for regex
				if (/^\s*$|[-+=;(/[,}]\s*$/.test(contents.substr(0, i))) {
					for (i++; i < contents.length && contents[i] !== '/'; i++) {
						if (contents[i] === '\\')
							i++;
					}
				}
			}
		} else if (contents[i] === '"' || contents[i] === "'") {
			var quot = contents[i];
			i++;

			while (i < contents.length && contents[i] !== quot) {
				if (contents[i] === '\\')
					i++;
				i++;
			}
		}

		return i;
	}

	function skipToOpenBrace(i) {
		while (i < contents.length && contents[i] !== '{')
			i++;
		return i;
	}

	function tryParseTag(i) {
		var match = istag(i);

		if (match) {
			newContents += contents.substr(lastCopied, i - lastCopied);

			i += match[0].length;

			lastCopied = i;

			var args = match[1].trim().replace(/\\\*.+?\*\\/, '');

			if (args) {
				var argsString = args ? args.split(/,/g).map(addQuotes).join('') : '';
				newContents += '[' + argsString + 'function (' + match[1] + ')';

			} else {
				newContents += 'function (' + match[1] + ')';
			}

			i = skipToOpenBrace(i) + 1;
			i = skipToFunctionEnd(i);

			if (lastCopied < i)
				newContents += contents.substr(lastCopied, i - lastCopied);

			lastCopied = i;

			if (args)
				newContents += ']';
		}

		return i;
	}

	function skipToFunctionEnd(i) {
		var indent = 1;

		for (; i < contents.length && indent > 0; i++) {
			i = skipStringsAndCommentsAndRegexes(i);

			if (contents[i] === '{')
				indent++;
			else if (contents[i] === '}')
				indent--;

			i = tryParseTag(i);
		}

		return i;
	}

	for (var i = 0; i < contents.length; i++) {
		i = skipStringsAndCommentsAndRegexes(i);
		i = tryParseTag(i);
	}

	if (lastCopied < contents.length)
		newContents += contents.substr(lastCopied);

	return newContents;
};