var expect = require('chai').expect
	, minifyFix = require('../index');

function f(code) {
	return minifyFix('a', 'b', code);
}

describe("ag-requirejs-angular-minify-fix", function () {
	it("should not change file without $ng tag", function () {
		expect(f("function foo() {}")).equal("function foo() {}");
	});

	it("should not change file if tag is not function name", function () {
		expect(f("function foo() { $ng }")).equal("function foo() { $ng }");
	});

	it("should not change file if tag is only part of function name", function () {
		expect(f("function $ngx() { }")).equal("function $ngx() { }");
	});

	it("should properly remove tag for function without arguments", function () {
		expect(f("function $ng() { }")).equal("function () { }");
	});

	it("should properly remove tag for function without arguments (semicolon)", function () {
		expect(f("function $ng() { };")).equal("function () { };");
	});

	it("should properly remove tag and pack function into array for 1 argument", function () {
		expect(f("function $ng($scope) { }")).equal('["$scope", function ($scope) { }]');
	});

	it("should properly remove tag and pack function into array for 1 argument (semicolon)", function () {
		expect(f("function $ng($scope) { };")).equal('["$scope", function ($scope) { }];');
	});

	it("should properly remove handle new lines", function () {
		expect(f("function \n$ng\n(\n$scope)\n { }")).equal('["$scope", function (\n$scope)\n { }]');
	});

	it("should properly remove tag and pack function into array for 3 arguments", function () {
		expect(f("function $ng($scope, a, b) { }")).equal('["$scope", "a", "b", function ($scope, a, b) { }]');
	});

	it("should ignore any whitespaces", function () {
		expect(f("function   $ng  (  $scope,  a, b  )  { }")).equal('["$scope", "a", "b", function (  $scope,  a, b  )  { }]');
	});

	it("should handle scopes properly", function () {
		expect(f("function () { function $ng($scope) { if (true) { } } }")).equal('function () { ["$scope", function ($scope) { if (true) { } }] }');
	});

	it("should ignore braces in comments /**/", function () {
		expect(f("function () { function $ng($scope) { if (true) { } /*}*/ } }")).equal('function () { ["$scope", function ($scope) { if (true) { } /*}*/ }] }');
	});

	it("should ignore braces in comments //", function () {
		expect(f("function () { function $ng($scope) { if (true) { } //}\n } }")).equal('function () { ["$scope", function ($scope) { if (true) { } //}\n }] }');
	});

	it("should ignore braces in regexes", function () {
		expect(f("function () { function $ng($scope) { if (true) { } /}/;\n } }")).equal('function () { ["$scope", function ($scope) { if (true) { } /}/;\n }] }');
	});
	
	it("should ignore $ng tag in comments", function () {
		expect(f("function /* $ng */($scope) { }")).equal('function /* $ng */($scope) { }');
	});

	it("should ignore $ng tagged function in comments /**/", function () {
		expect(f("/*function $ng($scope) { }*/")).equal('/*function $ng($scope) { }*/');
	});

	it("should ignore $ng tagged function in comments //", function () {
		expect(f("//function $ng($scope) { }")).equal('//function $ng($scope) { }');
	});

	it("should properly handle 2 $ng functions", function () {
		expect(f("function $ng($scope) { } function $ng(a) { }")).equal('["$scope", function ($scope) { }] ["a", function (a) { }]');
	});

	it("should properly handle $ng function inside another $ng function", function () {
		expect(f("function $ng($scope) { function $ng(a) { } }")).equal('["$scope", function ($scope) { ["a", function (a) { }] }]');
	});
});